{=========================================================================
  RkPanel Unit

  This unit implements an enhanced panel component which provides the
  ability to toggle individual sides of the panel, display multi-line
  captions, and fill in the area between bevels with a separate color.

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkPanel;

interface

uses
  SysUtils, Windows, Classes, Graphics, Controls, ExtCtrls,
  Menus, RkCommon;

type
  TSide = ( sdLeft, sdTop, sdRight, sdBottom );
  TSides = set of TSide;
  TVerticalAlignment = ( vaTop, vaCenter, vaBottom );

  TRkPanel = class( TCustomPanel )
  private
    FAboutInfo : TRkAboutInfo;
    FBevelSides : TSides;
    FBorderColor : TColor;
    FBorderSides : TSides;
    FVerticalAlignment : TVerticalAlignment;
    procedure SetBevelSides( Value : TSides );
    procedure SetBorderColor( Value : TColor );
    procedure SetBorderSides( Value : TSides );
    procedure SetVerticalAlignment( Value : TVerticalAlignment );
  protected
    procedure DrawSides( Rect : TRect; ULColor, LRColor : TColor;
                         Sides : TSides );
    procedure DrawBevel( var Rect : TRect; ULColor, LRColor : TColor;
                         Width : Integer; Sides : TSides );
    function GetClientRect : TRect; override;
    procedure Paint; override;
  public
    constructor Create( AOwner : TComponent ); override;
  published
    property About : TRkAboutInfo
      read FAboutInfo
      write FAboutInfo
      stored False;

    property BevelSides : TSides
      read FBevelSides
      write SetBevelSides
      default [ sdLeft, sdTop, sdRight, sdBottom ];

    property BorderColor : TColor
      read FBorderColor
      write SetBorderColor
      default clBtnFace;

    property BorderSides : TSides
      read FBorderSides
      write SetBorderSides;

    property VerticalAlignment : TVerticalAlignment
      read FVerticalAlignment
      write SetVerticalAlignment
      default vaCenter;

    { Inherited Properties & Events }
    property Align;
    property Alignment;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    (*
    property BorderStyle;  Property is Not redeclared to hide it from user
    *)
    property BorderWidth;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Caption;
    property Color;
    property Ctl3D;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;


implementation


{======================}
{== TRkPanel Methods ==}
{======================}

constructor TRkPanel.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
                         // Prevent Caption from being set to default name
  ControlStyle := ControlStyle - [ csSetCaption, csOpaque ];
                                                     // Set Default values
  FBevelSides := [ sdLeft, sdTop, sdRight, sdBottom ];
  FBorderColor := clBtnFace;
  FVerticalAlignment := vaCenter;
end;


procedure TRkPanel.SetBevelSides( Value : TSides );
begin
  if FBevelSides <> Value then
  begin
    FBevelSides := Value;
    Invalidate;
  end;
end;


procedure TRkPanel.SetBorderColor( Value : TColor );
begin
  if FBorderColor <> Value then
  begin
    FBorderColor := Value;
    Invalidate;
  end;
end;


procedure TRkPanel.SetBorderSides( Value : TSides );
begin
  if FBorderSides <> Value then
  begin
    FBorderSides := Value;
    Invalidate;
  end;
end;


procedure TRkPanel.SetVerticalAlignment( Value : TVerticalAlignment );
begin
  if FVerticalAlignment <> Value then
  begin
    FVerticalAlignment := Value;
    Invalidate;
  end;
end;


procedure TRkPanel.DrawSides( Rect : TRect; ULColor, LRColor : TColor;
                              Sides : TSides );
begin
  with Canvas, Rect do
  begin
    Pen.Color := ULColor;
    if sdLeft in Sides then
    begin
      MoveTo( Left, Top );
      LineTo( Left, Bottom );
    end;

    if sdTop in Sides then
    begin
      MoveTo( Left, Top );
      LineTo( Right, Top );
    end;

    Pen.Color := LRColor;
    if sdRight in Sides then
    begin
      MoveTo( Right - 1, Top );
      LineTo( Right - 1, Bottom );
    end;

    if sdBottom in Sides then
    begin
      MoveTo( Left, Bottom - 1 );
      LineTo( Right, Bottom - 1 );
    end;
  end;
end; {= TRkPanel.DrawSides =}



procedure TRkPanel.DrawBevel( var Rect : TRect; ULColor, LRColor : TColor;
                              Width : Integer; Sides : TSides );
var
  I : Integer;
begin
  Canvas.Pen.Width := 1;
  for I := 1 to Width do                    // Loop through width of bevel
  begin
    DrawSides( Rect, ULColor, LRColor, Sides );
    InflateRect( Rect, -1, -1 );
  end;
end;


function TRkPanel.GetClientRect : TRect;
begin
  Result := inherited GetClientRect;
  if sdLeft in FBorderSides then
    Inc( Result.Left );
  if sdTop in FBorderSides then
    Inc( Result.Top );
  if sdRight in FBorderSides then
    Dec( Result.Right );
  if sdBottom in FBorderSides then
    Dec( Result.Bottom );
end;


procedure TRkPanel.Paint;
var
  DrawRct, TextRct : TRect;
  TempStz    : array[ 0..255 ] of Char;
  H : Integer;
begin
  DrawRct := Rect( 0, 0, Width, Height );

  with Canvas, DrawRct do
  begin
    { Draw Border }

    DrawSides( DrawRct, clBlack, clBlack, FBorderSides );
    if sdLeft in FBorderSides then
      Inc( Left );
    if sdTop in FBorderSides then
      Inc( Top );
    if sdRight in FBorderSides then
      Dec( Right );
    if sdBottom in FBorderSides then
      Dec( Bottom );

    { Draw Bevels }

    if BevelOuter <> bvNone then
      DrawBevel( DrawRct, ULBevelColor[ BevelOuter ],
                 LRBevelColor[ BevelOuter ], BevelWidth, FBevelSides );

    if BorderWidth > 0 then
      DrawBevel( DrawRct, FBorderColor, FBorderColor, BorderWidth,
                 FBevelSides );

    if BevelInner <> bvNone then
      DrawBevel( DrawRct, ULBevelColor[ BevelInner ],
                 LRBevelColor[ BevelInner ], BevelWidth, FBevelSides );

    { Draw Caption }

    Brush.Color := Color;
    FillRect( DrawRct );    // Fill interior of panel to erase old caption
    Brush.Style := bsClear;

    Font := Self.Font;
    StrPCopy( TempStz, Caption );

    TextRct := DrawRct;
    H := DrawText( Handle, TempStz, -1, TextRct,
                   dt_CalcRect or dt_WordBreak or dt_ExpandTabs or
                   dt_VCenter or TextAlignments[ Alignment ] );

    if FVerticalAlignment = vaCenter then
    begin
      Top := ( ( Bottom + Top ) - H ) shr 1;
      Bottom := Top + H;
    end
    else if FVerticalAlignment = vaBottom then
      Top := Bottom - H - 1;

    DrawText( Handle, TempStz, -1, DrawRct,
              dt_WordBreak or dt_ExpandTabs or dt_VCenter or
              TextAlignments[ Alignment ] );
  end; { with }
end; {= TRkPanel.Paint =}


end.
