{=========================================================================
  RkPrgres Unit

  Custom progress bar component with several border styles (Flat, Ctl3D,
  and StatusBar).  Specialized properties (PartsComplete, TotalParts)
  makes it easy to control the progress bar during install processes, etc.

  Developing Custom Delphi 3 Components
  Copyright  1995-1997 by Ray Konopka
=========================================================================}

{$I DCDC.INC}

unit RkPrgres;

interface

uses
  Windows, Classes, Graphics, Controls, Menus, ExtCtrls, RkCommon;

type
  TProgressBorderStyle = ( bsFlat, bsCtl3D, bsStatusControl );
  TProgressOrientation = ( poHorizontal, poVertical );
  TPercentRange = 0..100;

  // Method pointer type for OnChange event
  TProgressChangeEvent = procedure( Sender : TObject;
                                    Percent : Integer ) of object;

  TRkProgressBar = class( TGraphicControl )
  private
    FAboutInfo : TRkAboutInfo;
    FBackColor : TColor;
    FBarColor : TColor;
    FBorderWidth : TBorderWidth;
    FBorderStyle : TProgressBorderStyle;
    FInteriorOffset : Byte;
    FOrientation : TProgressOrientation;
    FPartsComplete : Word;
    FPercent : TPercentRange;
    FShowPercent : Boolean;
    FTotalParts : Word;
    FOnChange : TProgressChangeEvent;

    procedure SetBackColor( Value : TColor );
    procedure SetBarColor( Value : TColor );
    procedure SetBorderWidth( Value : TBorderWidth );
    procedure SetBorderStyle( Value : TProgressBorderStyle );
    function GetCtl3D : Boolean;
    procedure SetCtl3D( Value : Boolean );
    procedure SetInteriorOffset( Value : Byte );
    procedure SetOrientation( Value : TProgressOrientation );
    procedure SetPartsComplete( Value : Word );
    procedure SetPercent( Value : TPercentRange );
    procedure SetShowPercent( Value : Boolean );
    procedure SetTotalParts( Value : Word );
  protected
    procedure Paint; override;
    procedure PercentChanged; dynamic;            // Event Dispatch Method
  public
    constructor Create( AOwner : TComponent ); override;
    procedure IncPartsByOne;
    procedure IncParts( N : Integer );
  published
    property About : TRkAboutInfo
      read FAboutInfo
      write FAboutInfo
      stored False;

    property BackColor : TColor
      read FBackColor
      write SetBackColor
      default clWhite;

    property BarColor : TColor
      read FBarColor
      write SetBarColor
      default clHighlight;

    property BorderStyle : TProgressBorderStyle
      read FBorderStyle
      write SetBorderStyle
      default bsCtl3D;

    property BorderWidth : TBorderWidth
      read FBorderWidth
      write SetBorderWidth;

    property Ctl3D : Boolean
      read GetCtl3D
      write SetCtl3D
      default True;

    property InteriorOffset : Byte
      read FInteriorOffset
      write SetInteriorOffset;

    property Orientation : TProgressOrientation
      read FOrientation
      write SetOrientation
      default poHorizontal;

    property PartsComplete : Word
      read FPartsComplete
      write SetPartsComplete;

    property Percent : TPercentRange
      read FPercent
      write SetPercent;

    property ShowPercent : Boolean
      read FShowPercent
      write SetShowPercent
      default True;

    property TotalParts : Word
      read FTotalParts
      write SetTotalParts;

    property OnChange : TProgressChangeEvent
      read FOnChange
      write FOnChange;

    { Inherited Properties & Events }
    property Align;
    property Color;
    property Font;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


procedure DrawPercentBar( Canvas : TCanvas; DrawRct : TRect;
                          Offset, InteriorOffset, BorderWidth : Integer;
                          Orientation : TProgressOrientation;
                          BarColor, BackColor : TColor;
                          Percent : Integer; ShowPercent : Boolean );


implementation

uses
  SysUtils;


{============================}
{== TRkProgressBar Methods ==}
{============================}

constructor TRkProgressBar.Create( AOwner : TComponent );
begin
  inherited Create( AOwner );
  ControlStyle := ControlStyle + [ csOpaque ];
  FPercent := 0;
  FShowPercent := True;
  FOrientation := poHorizontal;
  FBackColor := clWhite;
  FBarColor := clHighlight;
  FBorderWidth := 0;
  FBorderStyle := bsCtl3D;
  Width := 100;
  Height := 24;
end;



procedure TRkProgressBar.SetBackColor( Value : TColor );
begin
  if FBackColor <> Value then
  begin
    FBackColor := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetBarColor( Value : TColor );
begin
  if FBarColor <> Value then
  begin
    FBarColor := Value;
    Font.Color := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetBorderWidth( Value : TBorderWidth );
begin
  if FBorderWidth <> Value then
  begin
    FBorderWidth := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetBorderStyle( Value : TProgressBorderStyle );
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    if ( FBorderStyle = bsStatusControl ) and ( FBorderWidth = 0 ) then
      FBorderWidth := 2
    else if FBorderWidth = 2 then
      FBorderWidth := 0;
    Invalidate;
  end;
end;


function TRkProgressBar.GetCtl3D : Boolean;
begin
  Result := FBorderStyle = bsCtl3D;
end;


procedure TRkProgressBar.SetCtl3D( Value : Boolean );
begin
  if Ctl3D <> Value then
  begin
    if Value then
      FBorderStyle := bsCtl3D
    else
      FBorderStyle := bsFlat;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetInteriorOffset( Value : Byte );
begin
  if FInteriorOffset <> Value then
  begin
    FInteriorOffset := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetOrientation( Value : TProgressOrientation );
begin
  if FOrientation <> Value then
  begin
    FOrientation := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetPartsComplete( Value : Word );
begin
  if FPartsComplete <> Value then
  begin
    if Value > FTotalParts then
      FPartsComplete := FTotalParts
    else
      FPartsComplete := Value;

    // Setting the Percent property causes the SetPercent method to get
    // called  which will force a repaint
    Percent := Round( FPartsComplete / FTotalParts * 100 );
  end;
end;


procedure TRkProgressBar.SetPercent( Value : TPercentRange );
begin
  if FPercent <> Value then
  begin
    FPercent := Value;
    PercentChanged;

    // Call Repaint rather than Invalidate so that the view of the
    // component does not get erased.  This prevents flicker.
    Repaint;
  end;
end;


procedure TRkProgressBar.SetShowPercent( Value : Boolean );
begin
  if FShowPercent <> Value then
  begin
    FShowPercent := Value;
    Invalidate;
  end;
end;


procedure TRkProgressBar.SetTotalParts( Value : Word );
begin
  if FTotalParts <> Value then
  begin
    FTotalParts := Value;
    FPartsComplete := 0;              // Set internal FPartsComplete field
    Percent := 0;                                // Set Percent *property*
  end;
end;


procedure TRkProgressBar.IncPartsByOne;
begin
  IncParts( 1 );
end;


procedure TRkProgressBar.IncParts( N : Integer );
begin
  PartsComplete := PartsComplete + N;
end;


procedure TRkProgressBar.PercentChanged;
begin
  if Assigned( FOnChange ) then
    FOnChange( Self, FPercent );
end;


{=========================================================================
  DrawPercentBar

  This is an exported procedure to facilitate usage by other components
=========================================================================}

procedure DrawPercentBar( Canvas : TCanvas; DrawRct : TRect;
                          Offset, InteriorOffset, BorderWidth : Integer;
                          Orientation : TProgressOrientation;
                          BarColor, BackColor : TColor;
                          Percent : Integer; ShowPercent : Boolean );
var
  PercentStr : string;
  PctRct : TRect;
  TopOffset : Integer;
begin
  with Canvas, DrawRct do
  begin
    if ShowPercent then
      PercentStr := Format( '%u%%', [ Percent ] )
    else
      PercentStr := '';

    Font.Color := BackColor;
    Brush.Color := BarColor;

    // Calculate the size of the left/bottom portion of the percentage bar

    if Orientation = poVertical then
    begin
      PctRct := Rect( BorderWidth + Offset,
                      ( Bottom - Offset - Offset - BorderWidth ) -
                        Round((Longint(Bottom-Offset-Offset-BorderWidth) *
                              Percent ) / 100 ) + Offset + BorderWidth,
                      Right - Offset,
                      Bottom - Offset );
    end
    else
    begin
      PctRct := Rect( BorderWidth + Offset,
                      BorderWidth + Offset,
                      Round((Longint(Right-Offset-Offset - BorderWidth) *
                            Percent ) / 100 ) + Offset + BorderWidth,
                      Bottom - Offset );
    end;

    { Display the left/bottom portion of the percentage bar }

    SetTextAlign( Handle, ta_Center or ta_Top );
    TopOffset := Abs( Offset - 2 - InteriorOffset );
    TextRect( PctRct,  Right div 2,
              (Bottom-TextHeight('X')) div 2 + TopOffset, PercentStr );

    { Calculate the size of the right/top portion of the percentage bar }

    if Orientation = poVertical then
    begin
      PctRct.Bottom := PctRct.Top;
      PctRct.Top := Top + Offset;
    end
    else
    begin
      PctRct.Left := PctRct.Right;
      PctRct.Right := Right - Offset;
    end;

    { Display the right/top portion of the percentage bar }

    Font.Color := BarColor;
    Brush.Color := BackColor;
    TextRect( PctRct,  Right div 2,
              (Bottom-TextHeight('X')) div 2 + TopOffset, PercentStr );
  end; { with }
end; {= DrawPercentBar =}


procedure TRkProgressBar.Paint;
var
  Offset : Integer;
  DrawRct : TRect;
begin
  Canvas.Font := Font;
  DrawRct := ClientRect;
  InflateRect( DrawRct, -FBorderWidth, -FBorderWidth );

  with DrawRct, Canvas do
  begin
    { Draw Border }
    { DrawCtl3DBorder and DrawBorder are defined in the RkCommon unit }

    Offset := 1;
    case FBorderStyle of
      bsCtl3D:
      begin
        Inc( Offset );
        DrawCtl3DBorder( Canvas, DrawRct );
      end;

      bsStatusControl:
        DrawBorder( Canvas, DrawRct, fsRecessed );

      bsFlat:
        DrawBorder( Canvas, DrawRct, fsFlat )
    end; { case }

    { Draw Interior Offset Region }

    if FInteriorOffset > 0 then
    begin
      Brush.Style := bsClear;
      Pen.Color := Color;
      Pen.Style := psInsideFrame;
      Pen.Width := FInteriorOffset;
      Rectangle( Left + Offset, Top + Offset,
                 Right - Offset, Bottom - Offset );
      Pen.Width := 1;                      // Be sure to restore Pen width
      Pen.Style := psSolid;
      Brush.Style := bsSolid;
    end;
    Offset := Offset + FInteriorOffset;

    { Draw Percent Bar }

    DrawPercentBar( Canvas, DrawRct, Offset, FInteriorOffset,
                    FBorderWidth, FOrientation, FBarColor, FBackColor,
                    FPercent, FShowPercent );
  end; { with }
end; {= TRkProgressBar.Paint =}


end.

